<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gasecminutes\Administrator\Helper;

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\CMS\Mail\MailTemplate;
use \Joomla\CMS\Mail\MailerFactoryInterface;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GasecminutesHelper;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GamodalHelper;

/**
 * Gasecminutes helper.
 * @since  1.6
 */
class GaemailHelper
{

	/**
	* Notify the user/client/member etc through email
	* @param   array   $data passed by booking screen
	* @param   string  $tmpl  mandatory (template ext)
	* @param   integer $id booking reference if existing
	* @return true
	*/
	public static function sendNotification($data, $tmpl)
	{
		if ($data['id']) {
    		$params  = ComponentHelper::getParams('com_gasecminutes');
    		
    		$item = GasecminutesHelper::getRecord($data['id']);

    		// setup the data to include in email
    		$data['tran_ref'] = $item->tran_ref;
    		$data['start_date'] = HtmlHelper::date($item->start_date, Text::_('COM_GASECMINUTES_DISPLAY_DATE'));

            $data['recips'] = array(array('email'=>$item->user_email, 'name'=>$item->user_name));
    		$data['cc_recips'] = array(array('email'=>$item->user_email, 'name'=>$item->user_name));

			$viewLink = GamodalHelper::getHTTPQuery(null, 'view', 'meeting', 'id', $item->id);
			$link = 'index.php?'.http_build_query($viewLink, '', '&amp;');

            if ($params->get('tmpl_email', 0)) {
                self::sendEmailTemplate('com_gasecminutes.'.$tmpl, $data, null, $link, null, $params);
            } else {
                self::sendEmail($data['recips'], Text::_('COM_GASECMINUTES_EMAIL_BODY'), Text::_('COM_GASECMINUTES_EMAIL_SUBJECT'), null, $params);
            }
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_GASECMINUTES_NO_ID_MESSAGE'), 'warning');
        }

		return true;
	}

	/**
	 * Send out an email
	 * @param   array   $recipients  mandatory
	 * @param   string  $body  - text of the email body
	 * @param   string  $subject  - text for the subject line
	 * @param   string  $attachfile - file location address
	 * @param   array   $params
	 * @return  boolean	false on fail
	 */
    public static function sendEmail($recipients, $body, $subject, $attachfile, $params)
	{
	    $app		= Factory::getApplication();
        $mailfrom	= $app->get('mailfrom');       // system email address
        $fromname	= $app->get('fromname');       // Site name or system name
        // Build the email and send
        $mail = Factory::getContainer()->get(MailerFactoryInterface::class)->createMailer();
        //$mail = Factory::getMailer();
        $mail->isHTML(true);
		$mail->addRecipient($recipients);
		$mail->setSender(array($mailfrom, $fromname));
		$mail->setSubject($subject);
		$mail->setBody($body);
		if (is_file($attachfile)) {
            $mail->addAttachment($attachfile);
        }

        if ($params->get('send_email', 1)) {
            $sent = $mail->Send();
        }

        return true;
	}

	/**
	 * Send out an email using the template layout
	 * @param   string  $template_id  mandatory (full template_id ie com_gasales.mail)
	 * @param   array   $data  mandatory - containing all relevant info (mandatory elements recipients)
	 * @param   string  $filename  optional - name of the file if required without full path
	 * @param   string  $link  optional
	 * @param   string  $attachment  optional - including full path
	 * @return  boolean	false on fail
	 */
    public static function sendEmailTemplate($template_id, $data, $filename = null, $link = null, $attachfile = null, $params = null)
	{
	    $app		= Factory::getApplication();
        $mailfrom	= $app->get('mailfrom');
        $fromname	= $app->get('fromname');

        if ($link) {
            $mode = $app->get('force_ssl', 0) == 2 ? 1 : (-1);
            $data['link_text'] = Route::_($link, false, $mode);
        }

        // Build the email and send
        $mailer = new MailTemplate($template_id, $app->getLanguage()->getTag());
        $mailer->addTemplateData($data);

        // set the recipients with third param options ('to', 'cc', 'bcc')
        if (!empty($data['recips'])) {
            foreach ($data['recips'] as $recip) {
                $mailer->addRecipient($recip['email'], $recip['name'], 'to');
            }
        }

        if (!empty($data['cc_recips'])) {
            foreach ($data['cc_recips'] as $ccs) {
                $mailer->addRecipient($ccs['email'], $ccs['name'], 'cc');
            }
        }

        if (!empty($data['bcc_recips'])) {
            if (empty($data['recips'])) {
                $mailer->addRecipient($mailfrom, $fromname, 'to');
            }
            foreach ($data['bcc_recips'] as $bccs) {
                $mailer->addRecipient($bccs['email'], $bccs['name'], 'bcc');
            }
        }

        $mailer->setReplyTo($mailfrom, $fromname);

        if ($attachfile) {
            $mailer->addAttachment($filename, $attachfile);
        }

        // Try to send the email.
        try {
            if ($params->get('send_email', 1)) {
                $return = $mailer->send();
            } else {
                $return = true;
            }
            Factory::getApplication()->enqueueMessage(Text::_('COM_GASECMINUTES_MAIL_SEND_SUCCESSFUL'), 'success');
        } catch (\Exception $exception) {
            Factory::getApplication()->enqueueMessage(Text::_($exception->getMessage()), 'warning');
            $return = false;
        }

        return $return;
	}

}

