<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gasecminutes\Administrator\Helper;

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\User\UserHelper;
use \Joomla\CMS\User\UserFactoryInterface;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Component\ComponentHelper;
use \Joomla\Filesystem\Path;
use \Joomla\Filesystem\File;
use \Joomla\Filesystem\Folder;
use \Joomla\CMS\User\User;
use \Joomla\CMS\Date\Date;
use \Joomla\CMS\Mail\MailTemplate;
use \Joomla\CMS\Access\Access;
use \Joomla\CMS\Installer\Installer;
use \Joomla\Session\SessionInterface;
use \Joomla\CMS\Secmin\SiteSecmin;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GapdfHelper;

/**
 * Main component helper.
 */
class GasecminutesHelper
{
    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------------  Regularly used for most components       --------------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

    /**
     * Prints out a variable value in human readable format
     */
    public static function print_r2($val){
        echo '<pre>Test<br />';
        \print_r($val);
        echo  '</pre>';
    }

    /**
     * Gets the user record for the specific id reference
     */
    public static function getSpecificUser($id = 0)
	{
		if ($id) {
			$container = Factory::getContainer();
			$userFactory = $container->get(UserFactoryInterface::class);
			$user = $userFactory->loadUserById($id);
			//$user = Factory::getContainer()->get(UserFactoryInterface::class)->loadUserById($id);
		} else {
			$user = Factory::getApplication()->getIdentity();
		}
		unset($user->password);
		unset($user->activation);
		unset($user->params);
		unset($user->lastResetTime);
		unset($user->resetCount);
		unset($user->otpKey);
		unset($user->otep);
		unset($user->requireReset);

		return $user;
	}

	/**
	 * Set up a web asset object for use
	 */
	public static function setupGAWebApp()
	{
        // Boot the DI container.
        $container = Factory::getContainer();
        // Alias the session service key to the web session service.
        $container->alias(SessionInterface::class, 'session.web.site');
        // Get the secmin.
        $app      = $container->get(SiteSecmin::class);
        //$template = $app->getTemplate(true);
        //$params   = $template->params;
        //$wa = Factory::getApplication()->getDocument()->getWebAssetManager();
        
        return $app;
	}

    /**
     * Load template over-rides when using modal view
     * This is important for Cloud White template scheme
     * because modals have the primary css over-ride any
     * css loaded in the template index.php file
     */
    public static function loadTmplStyleModal($wa)
	{
        $tmpl = Factory::getApplication()->getTemplate(true);
        if ($tmpl->params->get('colorName') == 'colors_white') {
        	$wa->addInlineStyle('.btn-primary {background-color:'.$tmpl->params->get('btnPbgColor').' !important;}');
        	$wa->addInlineStyle('.nav-link {color:var(--template-contrast) !important;}');
        	$wa->addInlineStyle('.form-check-input:checked, .form-select[multiple] option:checked, [multiple].custom-select option:checked {background-color:'.$tmpl->params->get('btnPbgColor').' !important;}');
        	$wa->addInlineStyle('.page-item.active .page-link {color: var(--rkic41site-color-link); background-color: #c1cee1 !important; border-color: #dfe3e7 !important;}');
        	$wa->addInlineStyle('.form-select[multiple] option:checked, [multiple].custom-select option:checked {background-color: var(--rkic41site-color-primary-border) !important;}');
        }
		return true;
	}

	/**
	 * Get the version of the component
	 * @return version element of manifest
	 */
	public static function getComponentVersion()
	{
		$componentXML = Installer::parseXMLInstallFile(Path::clean(JPATH_ADMINISTRATOR . '/components/com_gasecminutes/gasecminutes.xml'));

		return $componentXML['version'];
	}

    /**
     * Gets the edit permission for an user
     * @param   mixed  $item  The item
     * @return  bool
     */
    public static function canUserEdit($user, $item)
    {
        $permission = false;

        if ($user->authorise('core.edit', 'com_gasecminutes')) {
            $permission = true;
        } else {
            if (isset($item->created_by) && $item->created_by) {
                if ($user->authorise('core.edit.own', 'com_gasecminutes') && $item->created_by == $user->id) {
                    $permission = true;
                }
            } else {
                if ($user->authorise('core.create', 'com_gasecminutes')) {
                    $permission = true;
                }
            }
        }

        return $permission;
    }

	/**
	 * Get the Model from another component for use
	 * @param   string  $name    The model name.
	 * @param   string  $prefix  The class prefix.
	 * @param   array   $config  Configuration array for model.
	 * @return object	The model
	 */
	public static function getForeignModel($name = 'Form', $prefix = 'Site', $config = array('ignore_request' => true))
	{
		$fmodel = Factory::getApplication()->bootComponent('com_content')->getMVCFactory()->createModel($name, $prefix, $config);

		return $fmodel;
	}

	/**
	 * Create a new record based on params
	 * @param   string  $table  Table name.
	 * @param   string  $key    The reference key
	 * @param   string  $value  Key value tyo be loaded
	 * @return boolean	false on failure
	 */
	public static function createNewRecord($table, $key, $value, $params)
	{
        $state = $params->get('auto_create', 0) ? 1 : 0;
        $new_rec = new \stdClass();
        $new_rec->id = 0;
        $new_rec->state = $state;
        $new_rec->trip_id = $key;
        $new_rec->cat_id = $value;
        $new_rec->approved_by = $value;
        $new_rec->comment = 'Auto created.';

		try {
			$result = Factory::getContainer()->get('DatabaseDriver')->insertObject($table, $new_rec);
			Factory::getApplication()->enqueueMessage(Text::_('COM_GASECMINUTES_NEWREC_SUCCESSFUL'), 'notice');
		} catch (RuntimeException $e) {
			Factory::getApplication()->enqueueMessage($e->getMessage().' '.Text::_('COM_GASECMINUTES_NEWREC_FAILED'), 'danger');
			return false;
		}
		return true;
	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------------  Get options for selection fields etc     --------------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

	public static function getListOptions($table = '#__gasecminutes_meetings', $label = 'Meeting', $fld_value = 'id', $fld_text = 'title' )
	{
        // setup for other languages regarding the selection
        $sel_label = strtoupper('COM_GASECMINUTES_SELECT_'.$label);
        $sel_label = Text::_($sel_label);
		// get the records
        $db = Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", "'.$sel_label.'" as "text" UNION SELECT '.$db->quoteName($fld_value).' as value, '.$db->quoteName($fld_text).' as text ');
		$query->from( $db->quoteName($table) );
		if ($table == '#__users') {
            $query->where(' block = 0' );
        } else {
            $query->where(' state = 1' );
        }
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getCategoryOptions($ext = 'com_gasecminutes.meetings', $label = 'Meeting Type', $field = 'title' )
	{
		// get the categories
        $db = Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select '.$label.' - " as "text" UNION SELECT id as value, '.$field.' as text ');
		$query->from( ' #__categories ' );
		$query->where(' published = 1' );
		$query->where(' extension = '.$db->quote($ext) );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}

	public static function getSecminOptions()
	{
        $db = Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - Select Minute Type - " as "text" UNION SELECT a.id as value, c.title as text ');
		$query->from( ' #__gasecminutes_secmins as a' );
		$query->join('LEFT', ' #__categories AS c ON c.id = a.cat_id');
		$query->where(' a.state = 1' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        $options = $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

		return $options;
	}
    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------------  Get some core reference type data elements   ----------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

    /**
    *   Method to get values for a member based on field id
    *   @param int $id field_id reference
    *   @param int $userId user id reference
    *   @return object field name and value
    */
	public static function getCustomFieldValue($id = 0, $userId = 0)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' b.title As fieldName, a.value AS fieldValue ');
		$query->from(' #__fields_values AS a ');
		$query->join('LEFT', ' #__fields AS b ON b.id = a.field_id');
		$query->where(' a.field_id = '.(int) $id );
		$query->where(' a.item_id = '.(int) $userId );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObject();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' Field Values');
	        return false;
	    }

	}

    /**
    *   Method to get the profile fields available
    *   if no local profile plugin and thus no local suffix provided then only std profile elements are listed
    *   @return array of object records
    */
	public static function getProfileFieldsOptions()
	{
        $params = ComponentHelper::getParams('com_gasecminutes');
        $proflocal = $params->get('profile_suffix', '');
        $prof_key = 'profile'.$proflocal.'.';
        $keyLen = strlen($prof_key);
		$startkey = $keyLen+1;
		$sel_field = Text::_('COM_GASECMINUTES_SELECT_FIELD');

		// get the user records in the supplier group
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' "0" as "value", " - '.$sel_field.' - " as "text" UNION SELECT profile_key as value, substr(profile_key,'.$startkey.') as text ');
		$query->from( $db->quoteName('#__user_profiles') );
		$query->where(' substr(profile_key,1,'.$keyLen.') = '.$db->Quote($prof_key) );
		$query->group(' profile_key ' );
		$query->order(' text ASC ' );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObjectList();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage(),'danger');
	        return false;
	    }

	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------  Get basic Record/s providing a table name and other reference data ------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

    /**
    *   Method to get the required record
    *   @param string $table table name
    *   @param int $id id reference
    *   @return object record data
    */
	public static function getRecord($table = '', $id = 0)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from($db->quoteName($table, 'a'));
		$query->where(' a.id = '.(int) $id );
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObject();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' Record');
	        return false;
	    }

	}

    /**
    *   Method to get the required record
    *   @param string $table table name
    *   @param string $field field name to match on the id reference
    *   @param int $id id reference
    *   @return object record data
    */
	public static function getRecordList($table = '', $field = '', $id = 0)
	{
        $db		= Factory::getContainer()->get('DatabaseDriver');
		$query	= $db->getQuery(true);
        $query->clear();
		$query->select(' a.* ');
		$query->from($db->quoteName($table, 'a'));
		$query->where($db->quoteName($field).' = '.(int) $id );
		if ($table == '#__gasecminutes_secmins') {
            $query->select($db->quoteName('catId.title', 'cat_id_name'));
            $query->join('LEFT', $db->quoteName('#__categories', 'catId'), $db->quoteName('catId.id') . ' = ' . $db->quoteName('a.cat_id'));
        }
		$query->order(' a.ordering asc');
		$db->setQuery((string)$query);

	    try {
	        return $db->loadObjectList();
	    } catch (RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage($e->getMessage().' RecordList');
	        return false;
	    }

	}

    /** --------------------------------------------------------------------------------------------------   **/
    /** -----------------------  Some specific processes for this particular component  ------------------   **/
    /** --------------------------------------------------------------------------------------------------   **/

}

