<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die;


use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Layout\LayoutHelper;
use \Joomla\CMS\Language\Text;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GasecminutesHelper;

/** @var \Joomla\CMS\WebAsset\WebAssetManager $wa */
$wa = $this->document->getWebAssetManager();
$wa->useScript('table.columns')
    ->useScript('multiselect');
$wa->usePreset('com_gasecminutes.gasecminutespreset');

$user      = GasecminutesHelper::getSpecificUser();
$listOrder = $this->state->get('list.ordering', 'a.start_date');
$listDirn  = $this->state->get('list.direction', 'DESC');
$saveOrder = $listOrder == 'a.ordering';
$version = GasecminutesHelper::getComponentVersion();

if ($saveOrder && !empty($this->items)) {
	$saveOrderingUrl = 'index.php?option=com_gasecminutes&task=meetings.saveOrderAjax&tmpl=component&' . Session::getFormToken() . '=1';
    HTMLHelper::_('sortablelist.sortable', 'meetingList', 'adminForm', strtolower($listDirn), $saveOrderingUrl);
}

/*
echo '<pre>Test<br />';
print_r($this->items);
echo '</pre>';
*/
?>

<form action="<?php echo Route::_('index.php?option=com_gasecminutes&view=meetings'); ?>" method="post"
	  name="adminForm" id="adminForm">
	<div class="row">
		<div class="col-md-12">
			<div id="j-main-container" class="j-main-container">

            	<?php echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this)); ?>

                <?php if (empty($this->items)) : ?>
                    <div class="alert alert-info">
                        <span class="icon-info-circle" aria-hidden="true"></span><span class="visually-hidden"><?php echo Text::_('INFO'); ?></span>
                        <?php echo Text::_('JGLOBAL_NO_MATCHING_RESULTS'); ?>
                    </div>
                <?php else : ?>
    				<table class="table table-striped" id="meetingList">
        				<caption id="captionTable">
        					<?php echo Text::_('Version: '.$version); ?>
        				</caption>
    					<thead>
    					<tr>
                            <td class="w-1 text-center">
                                <?php echo HTMLHelper::_('grid.checkall'); ?>
                            </td>
    						<?php if (isset($this->items[0]->ordering)): ?>
    							<th width="1%" class="nowrap center hidden-phone">
    	                            <?php echo HTMLHelper::_('searchtools.sort', '', 'a.ordering', $listDirn, $listOrder, null, 'asc', 'JGRID_HEADING_ORDERING', 'icon-menu-2'); ?>
    	                        </th>
    						<?php endif; ?>
    						<?php if (isset($this->items[0]->state)): ?>
    							<th width="1%" class="nowrap center">
    								<?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
    							</th>
    						<?php endif; ?>
    
    						<th class='left'>
    						<?php echo HTMLHelper::_('searchtools.sort',  'COM_GASECMINUTES_MEETINGS_TITLE', 'a.title', $listDirn, $listOrder); ?>
    						</th>
    						<th class='left'>
    						<?php echo HTMLHelper::_('searchtools.sort',  'COM_GASECMINUTES_MEETINGS_START_DATE', 'a.start_date', $listDirn, $listOrder); ?>
    						</th>
    						<th class='left'>
    						<?php echo HTMLHelper::_('searchtools.sort',  'COM_GASECMINUTES_MEETINGS_CAT_ID', 'cat_id_name', $listDirn, $listOrder); ?>
    						</th>
    						<th class='left'>
    						<?php echo HTMLHelper::_('searchtools.sort',  'COM_GASECMINUTES_CREATED_DATE', 'a.created_date', $listDirn, $listOrder); ?>
    						</th>
    						<th class='left'>
    						<?php echo HTMLHelper::_('searchtools.sort',  'COM_GASECMINUTES_ID', 'a.id', $listDirn, $listOrder); ?>
    						</th>

    						
    					</tr>
    					</thead>
    					<tfoot>
    					<tr>
    						<td colspan="<?php echo isset($this->items[0]) ? count(get_object_vars($this->items[0])) : 10; ?>">
    							<?php echo $this->pagination->getListFooter(); ?>
    						</td>
    					</tr>
    					</tfoot>
    					<tbody>
    					<?php foreach ($this->items as $i => $item) :
    						$ordering   = ($listOrder == 'a.ordering');
    						$canEdit    = GasecminutesHelper::canUserEdit($user, $item);
    						$canCreate  = $user->authorise('core.create', 'com_gasecminutes');
    						$canCheckin = $user->authorise('core.manage', 'com_gasecminutes');
    						$canChange  = $user->authorise('core.edit.state', 'com_gasecminutes');
    						?>
    						<tr class="row<?php echo $i % 2; ?>">
    
    							<td >
    								<?php echo HTMLHelper::_('grid.id', $i, $item->id); ?>
    							</td>
    							<?php if (isset($this->items[0]->ordering)) : ?>
    								<td class="order nowrap center hidden-phone">
    									<?php if ($canChange) :
    										$disableClassName = '';
    										$disabledLabel    = '';
    
    										if (!$saveOrder) {
    											$disabledLabel    = Text::_('JORDERINGDISABLED');
    											$disableClassName = 'inactive tip-top';
    										}
    									?>
    										<span class="sortable-handler hasTooltip <?php echo $disableClassName ?>"
    											  title="<?php echo $disabledLabel ?>">
    											<i class="icon-menu"></i>
    										</span>
    										<input type="text" style="display:none" name="order[]" size="5"
    											   value="<?php echo $item->ordering; ?>" class="width-20 text-area-order "/>
    									<?php else : ?>
    										<span class="sortable-handler inactive">
    											<i class="icon-menu"></i>
    										</span>
    									<?php endif; ?>
    								</td>
    							<?php endif; ?>
    							<?php if (isset($this->items[0]->state)): ?>
    								<td class="center">
    								<?php echo HTMLHelper::_('jgrid.published', $item->state, $i, 'meetings.', $canChange, 'cb'); ?>
    								</td>
    							<?php endif; ?>
    
    							<td>
    								<?php if (isset($item->checked_out) && $item->checked_out && ($canEdit || $canChange)) : ?>
    									<?php echo HTMLHelper::_('jgrid.checkedout', $i, $item->uEditor, $item->checked_out_time, 'meetings.', $canCheckin); ?>
    								<?php endif; ?>
    								<?php if ($canEdit) : ?>
    									<a href="<?php echo Route::_('index.php?option=com_gasecminutes&task=meeting.edit&id='.(int) $item->id); ?>">
    									<?php echo $this->escape($item->title); ?></a>
    								<?php else : ?>
    									<?php echo $this->escape($item->title); ?>
    								<?php endif; ?>
    							</td>
    							<td>
    								<?php $tdate = $item->start_date;
    									echo $tdate > 0 ? HTMLHelper::_('date', $tdate, Text::_('COM_GASECMINUTES_DISPLAY_DATE')) : '-'; ?>
    							</td>
    							<td><?php echo $item->cat_id_name; ?></td>
    							<td>
    								<?php $cdate = $item->created_date;
    									echo $cdate > 0 ? HTMLHelper::_('date', $cdate, Text::_('COM_GASECMINUTES_DISPLAY_DATE')) : '-'; ?>
    							</td>
    
    							<td><?php echo $item->id; ?></td>
    
    						</tr>
    					<?php endforeach; ?>
    					</tbody>
    				</table>
                <?php endif; ?>

				<input type="hidden" name="task" value=""/>
				<input type="hidden" name="boxchecked" value="0"/>
	            <input type="hidden" name="list[fullorder]" value="<?php echo $listOrder; ?> <?php echo $listDirn; ?>"/>
				<?php echo HTMLHelper::_('form.token'); ?>
			</div>
		</div>
	</div>
</form>
