<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gasecminutes\Site\Controller;

\defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Controller\BaseController;
use \Joomla\CMS\Router\Route;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GasecminutesHelper;

/**
 * Item class.
 * @since  1.6.0
 */
class SecminController extends BaseController
{
	/**
	 * Method to check out an item for editing and redirect to the edit form.
	 * @return void
	 * @since    1.6
     * @throws Exception
	 */
	public function edit()
	{
		$app = Factory::getApplication();

		// Get the previous edit id (if any) and the current edit id.
		$previousId = (int) $app->getUserState('com_gasecminutes.edit.secmin.id');
		$editId     = $app->input->getInt('id', 0);
		$meetId     = $app->input->getInt('meet_id', 0);

		// Set the record id to edit in the session.
		$app->setUserState('com_gasecminutes.edit.secmin.id', $editId);
		$app->setUserState('com_gasecminutes.edit.meeting.id', $meetId);

		// Get the model.
		$model = $this->getModel('Secmin', 'Site');

		// Check out the record
		if ($editId) {
			$model->checkout($editId);
		}

		// Check in the previous record.
		if ($previousId && $previousId !== $editId) {
			$model->checkin($previousId);
		}

		// Redirect to the edit screen.
		$this->setRedirect(Route::_('index.php?option=com_gasecminutes&view=secminform&layout=edit', false));
	}

	/**
	 * Method to publish record
	 * @return    void
	 * @throws Exception
	 * @since    1.6
	 */
	public function publish()
	{
		// Initialise variables.
		$app = Factory::getApplication();
		// Set up Redirect URL.
        $menu = $app->getMenu()->getActive();
        $url = (empty($menu->link) ? 'index.php?option=com_gasecminutes&view=secmins' : $menu->link.'&Itemid='.$menu->id);

		// Get the user data.
		$id    = $app->input->getInt('id');
		$state = $app->input->getInt('state');

		// Checking if the user can action the object
		$user = GasecminutesHelper::getSpecificUser();
		$item = GasecminutesHelper::getApplication($id);
		$canEdit = GasecminutesHelper::canUserEdit($user, $item);

		if ($canEdit)
		{
			$model = $this->getModel('Secmin', 'Site');

			// Attempt to save the data.
			$return = $model->publish($id, $state);

			// Check for errors.
			if ($return === false)
			{
				$app->enqueueMessage(Text::sprintf('COM_GASECMINUTES_SAVE_FAILED', $model->getError()), 'warning');
			}

			// Clear the record id from the session.
			$app->setUserState('com_gasecminutes.edit.secmin.id', null);

			// Flush the data from the session.
			$app->setUserState('com_gasecminutes.edit.secmin.data', null);

			// Redirect to the list screen.
			$app->enqueueMessage(Text::_('COM_GASECMINUTES_ITEM_SAVED_SUCCESSFULLY'), 'success');
		} else {
			throw new \Exception(500);
		}
		// Redirect to the list screen.
		$this->setRedirect(Route::_($url, false));
	}

	/**
	 * Remove data
	 * @return void
	 * @throws Exception
	 */
	public function remove()
	{
		// Initialise variables.
		$app = Factory::getApplication();
		// Set up Redirect URL.
        $item = $app->getMenu()->getActive();
        $url = (empty($item->link) ? 'index.php?option=com_gasecminutes&view=secmins' : $item->link.'&Itemid='.$item->id);

		// Checking if the user can remove object
		$user = GasecminutesHelper::getSpecificUser();

		if ($user->authorise('core.delete', 'com_gasecminutes'))
		{
			$model = $this->getModel('Secmin', 'Site');

			// Get the user submitted data.
			$id = $app->input->getInt('id', 0);

			// Attempt to save the data.
			$return = $model->delete($id);

			// Check for errors.
			if ($return === false) {
				$app->enqueueMessage(Text::sprintf(Text::_('COM_GASECMINUTES_ITEM_DELETED_FAILED'), $model->getError()), 'warning');
			} else {
				// Check in the record.
				if ($return) {
					$model->checkin($return);
				}

                $app->setUserState('com_gasecminutes.edit.secmin.id', null);
                $app->setUserState('com_gasecminutes.edit.secmin.data', null);

                $app->enqueueMessage(Text::_('COM_GASECMINUTES_ITEM_DELETED_SUCCESSFULLY'), 'success');
                $app->redirect(Route::_('index.php?option=com_gasecminutes&view=secmins', false));
			}

			// Redirect to the list screen.
			$this->setRedirect(Route::_($url, false));

		} else {
			throw new \Exception(500);
		}
	}
}
