<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Gasecminutes\Site\Model;

// No direct access.
defined('_JEXEC') or die;

use \Joomla\CMS\Factory;
use \Joomla\Utilities\ArrayHelper;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Table\Table;
use \Joomla\CMS\MVC\Model\ItemModel;
use \Joomla\CMS\Helper\TagsHelper;
use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Component\ComponentHelper;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GasecminutesHelper;

/**
 * Item model.
 * @since  1.6
 */
class MeetingModel extends ItemModel
{
    public $_item;

	/**
	 * Method to auto-populate the model state.
	 * Note. Calling getState in this method will result in recursion.
	 * @return void
	 * @since    1.6
     * @throws Exception
	 */
	protected function populateState()
	{
		$app  = Factory::getApplication('com_gasecminutes');
		$user = GasecminutesHelper::getSpecificUser();

		// Check published state
		if ((!$user->authorise('core.edit.state', 'com_gasecminutes')) && (!$user->authorise('core.edit', 'com_gasecminutes')))
		{
			$this->setState('filter.published', 1);
			$this->setState('filter.archived', 2);
		}

		// Load state from the request userState on edit or from the passed variable on default
		if (Factory::getApplication()->input->get('layout') == 'edit')
		{
			$id = Factory::getApplication()->getUserState('com_gasecminutes.edit.meeting.id');
		}
		else
		{
			$id = Factory::getApplication()->input->get('id');
			Factory::getApplication()->setUserState('com_gasecminutes.edit.meeting.id', $id);
		}

		$this->setState('meeting.id', $id);

		// Load the parameters.
		$params       = $app->getParams();
		$params_array = $params->toArray();

		if (isset($params_array['item_id'])) {
			$this->setState('meeting.id', $params_array['item_id']);
		}

		$this->setState('params', $params);
	}

	/**
	 * Method to get an object.
	 * @param   integer $id The id of the object to get.
	 * @return  mixed    Object on success, false on failure.
     * @throws Exception
	 */
	public function getItem($id = null)
	{
        if ($this->_item === null) {
            $this->_item = false;

            if (empty($id)) {
                $id = $this->getState('meeting.id');
            }

            // Get a venue row instance.
            $table = $this->getTable();

            // Attempt to load the row.
            if ($table->load($id)) {

                // Check published state.
                if ($published = $this->getState('filter.published')) {
                    if (isset($table->state) && $table->state != $published) {
                        throw new \Exception(Text::_('COM_GASECMINUTES_ITEM_NOT_LOADED'), 403);
                    }
                }

                // Convert the Table to a clean Object.
                // if backward compat plugin disabled then
                //    ArrayHelper::toObject($properties)
                // needs to be 
                //    ArrayHelper::toObject($properties, 'stdClass')
                // OR you can leave off the second param and it defaults to stdClass
                // Found in  libraries/vendor/joomla/utilities/src/ArrayHelper.php
                $properties  = $table->getProperties(1);
                $this->_item = ArrayHelper::toObject($properties, 'stdClass');

            }

            if (empty($this->_item)) {
				throw new \Exception(Text::_('COM_GASECMINUTES_ITEM_NOT_LOADED'), 404);
			}
        }

		if (isset($this->_item->created_by)) {
			$this->_item->created_by_name = GasecminutesHelper::getSpecificUser($this->_item->created_by)->name;
		}

		if (isset($this->_item->modified_by)) {
			$this->_item->modified_by_name = GasecminutesHelper::getSpecificUser($this->_item->modified_by)->name;
		}

		if (isset($this->_item->cat_id)) {
			$this->_item->cat_id_name = GasecminutesHelper::getRecord('#__categories', $this->_item->cat_id)->title;
		}

        return $this->_item;
    }

	/**
	 * Get an instance of Table class
	 * @param   string $type   Name of the Table class to get an instance of.
	 * @param   string $prefix Prefix for the table class name. Optional.
	 * @param   array  $config Array of configuration values for the Table object. Optional.
	 * @return  Table|bool Table if success, false on failure.
	 */
	public function getTable($type = 'Meeting', $prefix = 'Administrator', $config = array())
	{
		return parent::getTable($type, $prefix, $config);
	}
	public function getMinuteTable($type = 'Secmin', $prefix = 'Administrator', $config = array())
	{
		return parent::getTable($type, $prefix, $config);
	}

	/**
	 * Get the id of an item by alias
	 * @param   string $alias Item alias
	 * @return  mixed
	 */
	public function getItemIdByAlias($alias)
	{
        $table      = $this->getTable();
        $properties = $table->getProperties();
        $result     = null;
        $aliasKey   = null;
        if (method_exists($this, 'getAliasFieldNameByView')) {
            $aliasKey   = $this->getAliasFieldNameByView('meeting');
        }

        if (key_exists('alias', $properties)) {
            $table->load(array('alias' => $alias));
            $result = $table->id;
        } elseif (isset($aliasKey) && key_exists($aliasKey, $properties)) {
            $table->load(array($aliasKey => $alias));
            $result = $table->id;
        }

        return $result;

	}

	/**
	 * Method to check in an item.
	 * @param   integer $id The id of the row to check out.
	 * @return  boolean True on success, false on failure.
	 * @since    1.6
	 */
	public function checkin($id = null)
	{
		// Get the id.
		$id = (!empty($id)) ? $id : (int) $this->getState('meeting.id');
                
		if ($id) {
			// Initialise the table
			$table = $this->getTable();

			// Attempt to check the row in.
			if (method_exists($table, 'checkin')) {
				if (!$table->checkin($id)) {
					return false;
				}
			}
		}

		return true;
                
	}

	/**
	 * Method to check out an item for editing.
	 * @param   integer $id The id of the row to check out.
	 * @return  boolean True on success, false on failure.
	 * @since    1.6
	 */
	public function checkout($id = null)
	{
		// Get the user id.
		$id = (!empty($id)) ? $id : (int) $this->getState('meeting.id');

                
		if ($id) {
			// Initialise the table
			$table = $this->getTable();

			// Get the current user object.
			$user = GasecminutesHelper::getSpecificUser();

			// Attempt to check the row out.
			if (method_exists($table, 'checkout')) {
				if (!$table->checkout($user->id, $id)) {
					return false;
				}
			}
		}

		return true;
                
	}

	/**
	 * Publish the element
	 * @param   int $id    Item id
	 * @param   int $state Publish state
	 * @return  boolean
	 */
	public function publish($id, $state)
	{
		$table = $this->getTable();
                
		$table->load($id);
		$table->state = $state;

		return $table->store(true);
                
	}

	/**
	 * Method to delete an item
	 * @param   int $id Element id
	 * @return  bool
	 */
	public function delete($id)
	{
		$updateNulls = true;
		$table = $this->getTable();
		$table->load($id);
		$table->state = -2;

		return $table->store($updateNulls);
	}
	
	/**
	 * Method to push minutes to an article
	 * @param   int $id Element id
	 * @return  bool
	 */
	public function promote($id)
	{
		$app = Factory::getApplication();
        $lang = $app->getLanguage()->load('com_gasecminutes', JPATH_ADMINISTRATOR);
        $params = ComponentHelper::getParams('com_gasecminutes');
		$def_cat = $params->get('default_cat',2);
		$welcomTxt = $params->get('welcomTxt','');
        $artCat = GasecminutesHelper::getRecord('#__categories', $def_cat)->title;
		//$artModel = GasecminutesHelper::getForeignModel('Article', 'Administrator', ['ignore_request' => true]);
		$fcomp = $app->bootComponent('com_content')->getMVCFactory();
		$artModel =  $fcomp->createModel('Article', 'Administrator', ['ignore_request' => true]);
        $today = Factory::getDate()->toSql();

        $table = $this->getTable();
		$table->load($id);
        $table->state = 2;
        $table->store(true);
        
        $minRecs = GasecminutesHelper::getRecordList('#__gasecminutes_secmins', 'meet_id', $id);
        $meetCat = GasecminutesHelper::getRecord('#__categories', $table->cat_id)->title;
        $startDate = HTMLHelper::_('date', $table->start_date, Text::_('COM_GASECMINUTES_DISPLAY_DATE'));
        $startTime = HTMLHelper::_('date', $table->start_date, Text::_('COM_GASECMINUTES_DISPLAY_TIME'));
        $endTime = HTMLHelper::_('date', $table->end_date, Text::_('COM_GASECMINUTES_DISPLAY_TIME'));
        $art = [
            'introtext' => '<h4>'.$table->title.' at '.$table->venue.'</h4><div style="margin-left:30px;"><p><strong>Meeting Start Time: '.$startTime.'</strong></p><p>'.$welcomTxt.'</p>',
            'id' => 0,
            'catid' => $def_cat,
            'state' => 1,
            'title' => $meetCat.' - '.$startDate,
            'images' => '{"image_intro":"","image_intro_alt":"","float_intro":"","image_intro_caption":"","image_fulltext":"","image_fulltext_alt":"","float_fulltext":"","image_fulltext_caption":""}',
            'urls' => '{"urla":"","urlatext":"","targeta":"","urlb":"","urlbtext":"","targetb":"","urlc":"","urlctext":"","targetc":""}',
            'attribs' => '{"article_layout":"","show_title":"","link_titles":"","show_tags":"","show_intro":"","info_block_position":"","info_block_show_title":"","show_category":"","link_category":"","show_parent_category":"","link_parent_category":"","show_author":"","link_author":"","show_create_date":"","show_modify_date":"","show_publish_date":"","show_item_navigation":"","show_hits":"","show_noauth":"","urls_position":"","alternative_readmore":"","article_page_title":"","show_publishing_options":"","show_article_options":"","show_urls_images_backend":"","show_urls_images_frontend":""}',
            'created' => $today,
            'created_by' => $table->created_by,
            'modified' => $today,
            'modified_by' => $table->modified_by,
            'publish_up' => $today,
            'metadata' => '{"robots":"","author":"","rights":""}',
            'language' => '*',
        ];
        $art['introtext'] .= '<p><strong>Attendees: </strong>'.$table->attendees.'</p>';
        $art['introtext'] .= '<p><strong>Apologies: </strong>'.$table->apologies.'</p>';
        if ($table->guests > '') {
            $art['introtext'] .= '<p><strong>Guests: </strong>'.$table->guests.'</p>';
        }
        if ($table->newbies > '') {
            $art['introtext'] .= '<p><strong>New Members: </strong>'.$table->newbies.'</p>';
        }
        foreach ($minRecs as $m) {
            // add each minute to an article record
            $decision = $m->carried ? Text::_('COM_GASECMINUTES_CARRIED') : Text::_('COM_GASECMINUTES_REJECTED');
            $art['introtext'] .= '<blockquote style="margin-left:20px;"><p style="text-decoration:underline double;"><strong>'.$m->cat_id_name.'</strong></p>';
            $art['introtext'] .= $m->min_details;
            if ($m->voted) {
                $art['introtext'] .= '<p><strong>Moved: </strong>'.$m->moved;
                $art['introtext'] .= ' &nbsp; <strong>Seconder: </strong>'.$m->seconded;
                $art['introtext'] .= ' &nbsp; <strong>Decision: </strong>';
                $art['introtext'] .= $decision .'</p></blockquote>';
            } else {
                if (!empty($m->moved) && $m->moved > '') {
                    $art['introtext'] .= '<p><strong>Raised by: </strong>'.$m->moved.'</p></blockquote>';
                }
            }
            //  update the minute record
            $Mtable = $this->getMinuteTable();
            $Mtable->load($m->id);
            $Mtable->state = 2;
            $Mtable->store(true);
        }
        $art['introtext'] .= '<p><strong>Meeting Closed at: '.$endTime.'</strong></p></div>';
        
        try {
          $artModel->save($art);
          $app->enqueueMessage('No errors', 'message');
          return true;
        } catch(Exception $e) {
          $app->enqueueMessage($e->getMessage(), 'danger');
          return false;
        }

	}
	
}
