<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Uri\Uri;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GasecminutesHelper;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GamodalHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gasecminutes.gasecminutespreset');


// Load admin language file
$lang = Factory::getLanguage();
$lang->load('com_gasecminutes', JPATH_ADMINISTRATOR);

$user = GasecminutesHelper::getSpecificUser();
$canEdit = GasecminutesHelper::canUserEdit($user, $this->item);

$canDelete = $user->authorise('core.delete','com_gasecminutes.meeting.'.$this->item->id);

$articleTran = GamodalHelper::getHTTPQuery(null, 'task', 'meeting.promote', 'id', $this->item->id);
$articleURL = 'index.php?'.\http_build_query($articleTran, '', '&amp;');
$removeTran = GamodalHelper::getHTTPQuery(null, 'task', 'meeting.remove', 'id', $this->item->id);
$removeURL = 'index.php?'.\http_build_query($removeTran, '', '&amp;');
$editTran = GamodalHelper::getHTTPQuery(null, 'task', 'meeting.edit', 'id', $this->item->id);
$editURL = 'index.php?'.\http_build_query($editTran, '', '&amp;');
$canTran = GamodalHelper::getHTTPQuery(null, 'task', 'meeting.cancel', 'id', $this->item->id);
$canURL = 'index.php?'.\http_build_query($canTran, '', '&amp;');
$minURL = GamodalHelper::setupModalButton('view', 'secminform', 'id', 0, 'modalmin', 'myMinutes', 'success', 'COM_GASECMINUTES_TITLE_SECMINS', 'COM_GASECMINUTES_TITLE_SECMINS', 'icon-plus', '', 'meet_id', $this->item->id);
$minutes = GasecminutesHelper::getRecordList('#__gasecminutes_secmins', 'meet_id', $this->item->id);

//GasecminutesHelper::print_r2(Factory::getApplication()->getUserState('com_gasecminutes.test.data'));

// Display a heading for the page
if ($this->params->get('page_title', '') > '') {
	echo '<h1>'.$this->params->get('page_title').'</h1>';
} else { 
	echo '<h1>'.Text::_('COM_GASECMINUTES_TITLE_MEETING').'</h1>';
}

?>

<div class="item_fields">

	<table class="table">

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_MEETING_TITLE'); ?></th>
			<td><?php echo $this->item->title; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_MEETING_CAT_ID'); ?></th>
			<td><?php echo $this->item->cat_id_name; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_MEETING_START_DATE'); ?></th>
			<td><?php 
					$tdate = $this->item->start_date;
					echo $tdate > 0 ? HTMLHelper::_('date', $tdate, Text::_('COM_GASECMINUTES_DISPLAY_DATETIME')) : '-';
				?>
			</td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_TITLE_SECMINS'); ?></th>
			<td><?php echo \count($minutes); ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_MEETING_END_DATE'); ?></th>
			<td><?php 
					$edate = $this->item->end_date;
					echo $edate > 0 ? HTMLHelper::_('date', $edate, Text::_('COM_GASECMINUTES_DISPLAY_DATETIME')) : '-';
				?>
			</td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_MEETING_NEXT_DATE'); ?></th>
			<td><?php 
					$ndate = $this->item->next_date;
					echo $ndate > 0 ? HTMLHelper::_('date', $ndate, Text::_('COM_GASECMINUTES_DISPLAY_DATETIME')) : '-';
				?>
			</td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_CREATED_DATE'); ?></th>
			<td><?php echo $this->item->created_date; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_MODIFIED_DATE'); ?></th>
			<td><?php echo $this->item->modified_date; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_COMMENT'); ?></th>
			<td><?php echo nl2br($this->item->comment); ?></td>
		</tr>

	</table>

</div>

<?php /* ----------------------   Return button   ------------------------------ */ ?>
<a class="btn btn-secondary" href="<?php echo Route::_($canURL); ?>">
	<i class="icon-undo"></i> <?php echo Text::_("COM_GASECMINUTES_RETURN"); ?>
</a>

<?php /* ----------------------   Edit button   ------------------------------ */ ?>
<?php if($canEdit && $this->item->checked_out == 0 && $this->item->state != 2 ): ?>
	<a class="btn btn-warning" href="<?php echo Route::_($editURL); ?>">
		<i class="icon-edit"></i> <?php echo Text::_("COM_GASECMINUTES_EDIT_ITEM"); ?>
	</a>
<?php endif; ?>

<?php /* ----------------------   Minute button   ------------------------------ */ ?>
<?php if($canEdit && $this->item->checked_out == 0 && $this->item->state != 2): ?>
	<?php echo $minURL; ?>
	<?php if ($this->params->get('loadToArticle', 0)) : ?>
    <a class="btn btn-info" href="<?php echo Route::_($articleURL); ?>">
		<i class="icon-archive"></i> <?php echo Text::_("COM_GASECMINUTES_PROMOTE"); ?>
	</a>
	<?php endif; ?>
<?php endif; ?>

<?php /* ----------------------   Delete button and modal  ------------------------------ */ ?>
<?php if ($canDelete && $this->item->state != 2) : ?>
	<a href="<?php echo Route::_($removeURL, false, 2); ?>" class="btn btn-danger delete-button pull-right" type="button">
        <i class="icon-trash" ></i> <?php echo Text::_("COM_GASECMINUTES_DELETE_ITEM"); ?>
    </a>
    <script type="text/javascript">
    	jQuery(document).ready(function () {
    		jQuery('.delete-button').click(deleteItem);
    	});
    	function deleteItem() {
    		if (!confirm("<?php echo Text::_('COM_GASECMINUTES_DELETE_MESSAGE'); ?>")) {
    			return false;
    		}
    	}
    </script>
<?php endif; ?>

<div class="table-responsive com-contact-categories categories-list">
	<table class="table table-striped" id="secminList">
		<thead>
    		<tr>
    		<th class='left'>
    			<?php echo Text::_('COM_GASECMINUTES_SECMINS_CAT_ID'); ?>
    		</th>
    		<th class='left'>
    			<?php echo Text::_('COM_GASECMINUTES_SECMINS_MOVED'); ?>
    		</th>
    		<th class='left'>
    			<?php echo Text::_('COM_GASECMINUTES_SECMINS_SECONDED'); ?>
    		</th>
    		<th class='left'>
    			<?php echo Text::_('COM_GASECMINUTES_SECMINS_CARRIED'); ?>
    		</th>
    		</tr>
		</thead>
		<tbody>
    		<?php foreach ($minutes as $min) : ?>
    			<tr>
        			<td><?php echo GasecminutesHelper::getRecord('#__categories', $min->cat_id)->title; ?></td>
        			<td><?php echo $min->moved; ?></td>
        			<td><?php echo $min->seconded; ?></td>
        			<td><?php if ($min->voted) {echo $min->carried == 1 ? Text::_('COM_GASECMINUTES_CARRIED') : Text::_('COM_GASECMINUTES_REJECTED');} ?></td>
    			</tr>
    		<?php endforeach; ?>
		</tbody>
	</table>
</div>
