<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Uri\Uri;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GasecminutesHelper;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GamodalHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gasecminutes.gasecminutespreset');


// Load admin language file
$lang = Factory::getLanguage();
$lang->load('com_gasecminutes', JPATH_ADMINISTRATOR);

$user = GasecminutesHelper::getSpecificUser();
$canEdit = GasecminutesHelper::canUserEdit($user, $this->item);

$canDelete = $user->authorise('core.delete','com_gasecminutes.secmin.'.$this->item->id);

$removeTran = GamodalHelper::getHTTPQuery(null, 'task', 'secmin.remove', 'id', $this->item->id);
$removeURL = 'index.php?'.\http_build_query($removeTran, '', '&amp;');
$editTran = GamodalHelper::getHTTPQuery(null, 'task', 'secmin.edit', 'id', $this->item->id);
$editURL = 'index.php?'.\http_build_query($editTran, '', '&amp;');
$canTran = GamodalHelper::getHTTPQuery(null, 'task', 'secmin.cancel', 'id', $this->item->id);
$canURL = 'index.php?'.\http_build_query($canTran, '', '&amp;');

// Display a heading for the page
if ($this->params->get('page_title', '') > '') {
	echo '<h1>'.$this->params->get('page_title').'</h1>';
} else { 
	echo '<h1>'.Text::_('COM_GASECMINUTES_TITLE_SECMIN').'</h1>';
}
?>

<div class="item_fields">

	<table class="table">

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_SECMIN_MEET_ID'); ?></th>
			<td><?php echo $this->item->meet_id_name; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_SECMIN_CAT_ID'); ?></th>
			<td><?php echo $this->item->cat_id_name; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_MEETING_MIN_DETAILS'); ?></th>
			<td><?php echo $this->item->min_details; ?></td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_CREATED_DATE'); ?></th>
			<td><?php
					$tdate = $this->item->created_date;
					echo $tdate > 0 ? HTMLHelper::_('date', $tdate, Text::_('COM_GASECMINUTES_DISPLAY_DATETIME')) : '-';
				?>
			</td>
		</tr>

		<tr>
			<th><?php echo Text::_('COM_GASECMINUTES_FORM_LBL_COMMENT'); ?></th>
			<td><?php echo nl2br($this->item->comment); ?></td>
		</tr>

	</table>

</div>

<?php /* ----------------------   Return button   ------------------------------ */ ?>
<a class="btn btn-secondary" href="<?php echo Route::_($canURL); ?>">
	<i class="icon-undo"></i> <?php echo Text::_("COM_GASECMINUTES_RETURN"); ?>
</a>

<?php /* ----------------------   Edit button   ------------------------------ */ ?>
<?php if($canEdit && $this->item->checked_out == 0): ?>
	<a class="btn btn-warning" href="<?php echo Route::_($editURL); ?>">
		<i class="icon-edit"></i> <?php echo Text::_("COM_GASECMINUTES_EDIT_ITEM"); ?>
	</a>
<?php endif; ?>

<?php /* ----------------------   Delete button and modal  ------------------------------ */ ?>
<?php if ($canDelete) : ?>
	<a href="<?php echo Route::_($removeURL, false, 2); ?>" class="btn btn-danger delete-button" type="button">
        <i class="icon-trash" ></i>
    </a>
    <script type="text/javascript">
    	jQuery(document).ready(function () {
    		jQuery('.delete-button').click(deleteItem);
    	});
    	function deleteItem() {
    		if (!confirm("<?php echo Text::_('COM_GASECMINUTES_DELETE_MESSAGE'); ?>")) {
    			return false;
    		}
    	}
    </script>
<?php endif; ?>
