<?php
/**
 * @version    0.0.3
 * @package    com_gasecminutes
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

use \Joomla\CMS\HTML\HTMLHelper;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Uri\Uri;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Language;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\Layout\LayoutHelper;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GasecminutesHelper;
use \GlennArkell\Component\Gasecminutes\Administrator\Helper\GamodalHelper;

// load any assets required
$this->document->getWebAssetManager()
    ->usePreset('com_gasecminutes.gasecminutespreset');

// Load admin language file
$lang = Factory::getApplication()->getLanguage();
$lang->load('com_gasecminutes', JPATH_ADMINISTRATOR);

$listOrder  = $this->state->get('list.ordering','a.ordering');
$listDirn   = $this->state->get('list.direction','asc');

$user       = GasecminutesHelper::getSpecificUser();
$canCreate  = $user->authorise('core.create', 'com_gasecminutes');
$canEdit    = $user->authorise('core.edit', 'com_gasecminutes');
$canCheckin = $user->authorise('core.manage', 'com_gasecminutes');
$canChange  = $user->authorise('core.edit.state', 'com_gasecminutes');
$canEditOwn  = $user->authorise('core.edit.own', 'com_gasecminutes');
$canDelete  = $user->authorise('core.delete', 'com_gasecminutes');
$this->canMembers  = $user->authorise('core.members', 'com_gasecminutes');

$baseURL = 'index.php?';
$createTran = GamodalHelper::getHTTPQuery(null, 'task', 'secmin.edit', 'id', 0);
$createURL = $baseURL.\http_build_query($createTran, '', '&amp;');

/*
GasecminutesHelper::print_r2(Factory::getApplication()->getUserState('com_gasecminutess.test.data'))
*/

// Display a heading for the page
if ($this->params->get('page_title', '') > '') {
	echo '<div class="page-header"><h1 itemprop="headline">'.$this->params->get('page_title').'</h1></div>';
} else { 
	echo '<div class="page-header"><h1 itemprop="headline">'.Text::_('COM_GASECMINUTES_TITLE_SECMINS').'</h1></div>';
}
?>

<form action="<?php echo htmlspecialchars(Uri::getInstance()->toString()); ?>" method="post"
      name="adminForm" id="adminForm" class="com-content-category__articles">

	<?php
        // display standard joomla filter files
        //echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this));
        // display component view specific filter files
        echo LayoutHelper::render('default_filter', array('view' => $this), dirname(__FILE__));
    ?>

    <div class="table-responsive com-contact-categories categories-list">
	<table class="table table-striped" id="secminList">
		<thead>
		<tr>
			<?php if (isset($this->items[0]->state)): ?>
				<th width="5%">
					<?php echo HTMLHelper::_('grid.sort', 'JPUBLISHED', 'a.state', $listDirn, $listOrder); ?>
				</th>
			<?php endif; ?>

			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GASECMINUTES_MEETINGS_TITLE', 'meet_id_name', $listDirn, $listOrder); ?>
			</th>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GASECMINUTES_SECMINS_CAT_ID', 'cat_id_name', $listDirn, $listOrder); ?>
			</th>
    		<th class='left'>
    			<?php echo HTMLHelper::_('grid.sort',  'COM_GASECMINUTES_MEETINGS_CAT_ID', 'mcat_id_name', $listDirn, $listOrder); ?>
    		</th>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GASECMINUTES_CREATED_DATE', 'a.created_date', $listDirn, $listOrder); ?>
			</th>
			<th class=''>
				<?php echo HTMLHelper::_('grid.sort',  'COM_GASECMINUTES_ID', 'a.id', $listDirn, $listOrder); ?>
			</th>


			<?php if ($canEditOwn || $canCheckin || $canDelete): ?>
				<th class="center">
					<?php echo Text::_('COM_GASECMINUTES_ACTIONS'); ?>
				</th>
			<?php endif; ?>

		</tr>
		</thead>
		<tfoot>
		<tr>
			<td colspan="<?php echo isset($this->items[0]) ? count(get_object_vars($this->items[0])) : 10; ?>">
				<?php echo $this->pagination->getListFooter(); ?>
			</td>
		</tr>
		</tfoot>
		<tbody>
		<?php foreach ($this->items as $i => $item) : ?>
			<?php

                $canEdit = GasecminutesHelper::canUserEdit($user, $item);

                $linkTran = GamodalHelper::getHTTPQuery(null, 'view', 'secmin', 'id', $item->id);
                $appURL = $baseURL.\http_build_query($linkTran, '', '&amp;');

                $editTran = GamodalHelper::getHTTPQuery(null, 'task', 'secmin.edit', 'id', $item->id);
                $editURL = $baseURL.\http_build_query($editTran, '', '&amp;');

                $removeTran = GamodalHelper::getHTTPQuery(null, 'task', 'secmin.remove', 'id', $item->id);
                $removeURL = $baseURL.\http_build_query($removeTran, '', '&amp;');

                $archTran = GamodalHelper::getHTTPQuery(null, 'task', 'secmin.publish', 'id', $item->id);
                $archTran = GamodalHelper::getHTTPQuery($archTran, null, null, 'state', 2);
                $archURL = $baseURL.\http_build_query($archTran, '', '&amp;');

            ?>

			<tr class="row<?php echo $i % 2; ?>">

				<?php if (isset($this->items[0]->state)) : ?>
					<?php $class = ($canChange) ? 'active' : 'disabled'; ?>
					<td class="center">
						<a class="btn btn-micro <?php echo $class; ?>" href="<?php echo ($canChange) ? Route::_('index.php?option=com_gasecminutes&task=secmin.publish&id=' . $item->id . '&state=' . (($item->state + 1) % 2), false, 2) : '#'; ?>">
						<?php if ($item->state == 1): ?>
							<i class="icon-publish"></i>
						<?php else: ?>
							<i class="icon-unpublish"></i>
						<?php endif; ?>
						</a>
					</td>
				<?php endif; ?>

				<td>
					<?php if (isset($item->checked_out) && $item->checked_out) : ?>
						<?php echo HTMLHelper::_('jgrid.checkedout', $i, $item->uEditor, $item->checked_out_time, 'secmins.', $canCheckin); ?>
					<?php endif; ?>
					<a href="<?php echo Route::_($appURL); ?>">
						<?php echo $this->escape($item->meet_id_name); ?>
					</a>
				</td>
				<td>
					<?php echo $item->cat_id_name; ?>
				</td>
				<td>
					<?php echo $item->mcat_id_name; ?>
				</td>
				<td>
					<?php echo $item->created_date > 0 ? HTMLHelper::_('date', $item->created_date, Text::_('COM_GASECMINUTES_DISPLAY_DATE_ABRV')) : '-'; ?>
				</td>
				<td>
					<?php echo $item->id; ?>
				</td>

				<?php if ($canEditOwn || $canCheckin || $canDelete): ?>
					<td class="center">
						<a href="<?php echo Route::_($archURL); ?>" class="btn btn-success"
                            type="button" title="<?php echo Text::_('COM_GASECMINUTES_MARK_ARCHIVED'); ?>">
                            <i class="icon-redo" ></i>
                        </a>
						<?php if ($canEdit): ?>
							<a href="<?php echo Route::_($editURL); ?>" class="btn btn-warning" type="button">
                                <i class="icon-edit" ></i>
                            </a>
						<?php endif; ?>
						<?php if ($canDelete): ?>
							<a href="<?php echo Route::_($removeURL); ?>" class="btn btn-danger delete-button" type="button">
                                <i class="icon-trash" ></i>
                            </a>
						<?php endif; ?>
					</td>
				<?php endif; ?>

			</tr>
		<?php endforeach; ?>
		</tbody>
	</table>
        </div>
	<?php if ($canCreate) : ?>
		<a href="<?php echo Route::_($createURL); ?>" class="btn btn-success btn-small">
            <i class="icon-plus"></i> <?php echo Text::_('COM_GASECMINUTES_ADD_ITEM'); ?>
        </a>
	<?php endif; ?>

	<input type="hidden" name="task" value=""/>
	<input type="hidden" name="boxchecked" value="0"/>
	<input type="hidden" name="filter_order" value="<?php echo $listOrder; ?>"/>
	<input type="hidden" name="filter_order_Dir" value="<?php echo $listDirn; ?>"/>
	<?php echo HTMLHelper::_('form.token'); ?>
</form>

<?php if($canDelete) : ?>
<script type="text/javascript">

	jQuery(document).ready(function () {
		jQuery('.delete-button').click(deleteItem);
	});

	function deleteItem() {

		if (!confirm("<?php echo Text::_('COM_GASECMINUTES_DELETE_MESSAGE'); ?>")) {
			return false;
		}
	}
</script>
<?php endif; ?>
